package checkAlign;

public class Schneider {
	private double[] H;
	private double[] R;
	private double[] Rc;
	private double[][] W;
	
	private String tableLine;
	private double e;
	private double sd;
	private double hg;
	
	public Schneider(){
	}
	
	public void calculateSchneider(double[][] freqMatrix, boolean gaps, boolean correction, double e){
		// get number of symbols
		int numLetters = freqMatrix.length;
		// length of alignment
		int longSeq = freqMatrix[0].length;
		
		this.e = e;
		
		if (!gaps){
			if (numLetters == 5 || numLetters == 21){
				numLetters --;
			}
		}
		
		H = getHMatrix(freqMatrix);
		R = getRMatrix(H, numLetters, longSeq);	
		
		
		// hg value
		if (numLetters == 4){
			hg = 2;
		}else if(numLetters == 5){
			hg = 2.32;
		}else if(numLetters == 20){
			hg = 4.32;
		}else if(numLetters == 21){
			hg = 4.39;
		}
		
		if (correction){			
			Rc = getRcMatrix(H, e, hg);			
		}
		
		if (correction){
			W = getW(freqMatrix, Rc, hg);
		}else{
			W = getW(freqMatrix, R, hg);
		}
	}
	
	private double[] getHMatrix(double[][] freqMatrix){
		int numLetters = freqMatrix.length;
		int longSeq = freqMatrix[0].length;
		// result matrix
		double[] H = new double[longSeq];
		// summatory variable
		double sum = 0;		
		
		// each position
		for (int pos=0; pos < longSeq; pos++){
			// each letter
			sum = 0;
			for (int letter=0; letter<numLetters; letter++){
				if (freqMatrix[letter][pos] != 0){					
					sum += freqMatrix[letter][pos] * (Math.log(freqMatrix[letter][pos]) / Math.log(2));
				}
			}
			H[pos] = -(sum);
		}
		
		return H;
	}
	private double[] getRMatrix(double[] H, int numLetters, int longSeq){
		double[] R = new double[longSeq];
		
		for (int pos=0; pos < longSeq; pos++){			
			R[pos] = (Math.log(numLetters) / Math.log(2)) - H[pos];
		}
		
		return R;
	}
	private double[] getRcMatrix(double[] H, double e, double hg){
		double[] Rc = new double[H.length];
		
		for (int i=0; i<H.length; i++){
			Rc[i] = hg - (H[i] + e);
		}
		
		return Rc;
	}
	private double[][] getW(double[][]freqMatrix, double[]R, double hg){
		int numLetters = freqMatrix.length;
		int longSeq = freqMatrix[0].length;
		
		double[][] W = new double[numLetters][longSeq];
		
		
		for (int col=0; col<longSeq; col++){
			for (int letter=0; letter<numLetters; letter++){
				if (freqMatrix[letter][col] != 0){
					W[letter][col] = roundDecimals(((freqMatrix[letter][col] * R[col]) / hg), 2);
				}else{
					W[letter][col] = 0;
				}
			}
		}
		
		return W;
	}
	void showMatrix(double[][] matrix){
    	int numLetters = matrix.length;
    	int longSeq = matrix[0].length;
    	for (int i=0; i<numLetters; i++){
    		System.out.print(i+": ");
    		for (int j=0; j<longSeq; j++){
    			System.out.print(matrix[i][j]+" ");
    		}
    		System.out.print("\n");
    	}
    }
	private double roundDecimals(double num, int decimals){
    	//double res = ((int)(num*(Math.pow(10,decimals)))) / Math.pow(10,decimals);
    	double res = num * Math.pow(10, decimals);
    	res = Math.round(res);
    	res = res / Math.pow(10, decimals);
    	
    	return res;
    }
	public double[][] getMatrix(){
		return W;
	}	
	public double getHg(){
		return hg;
	}
	
}
